import argparse
import random

import parsl
from parsl.app.app import python_app
from parsl.tests.configs.local_threads import config


@python_app
def map_one(x, dur):
    import time
    time.sleep(dur)
    return x * 2


@python_app
def map_two(x, dur):
    import time
    time.sleep(dur)
    return x * 5


@python_app
def add_two(x, y, dur):
    import time
    time.sleep(dur)
    return x + y


def test_func_1(width=2):

    fu_1 = []
    for i in range(1, width + 1):
        fu = map_one(i, random.randint(0, 5) / 10)
        fu_1.extend([fu])

    fu_2 = []
    for fu in fu_1:
        fu = map_two(fu, 0)
        fu_2.extend([fu])

    assert sum([i.result() for i in fu_2]) == sum(
        range(1, width + 1)) * 10, "Sums do not match"
    return fu_2


def test_func_2(width=2):

    fu_1 = []
    for i in range(1, width + 1):
        fu = map_one(i, random.randint(0, 5))
        fu_1.extend([fu])

    fu_2 = []
    for i in range(0, width + 1, 2)[0:-1]:
        fu = add_two(fu_1[i], fu_1[i + 1], 0)
        fu_2.extend([fu])

    assert sum([i.result() for i in fu_2]) == sum(
        range(1, width + 1)) * 2, "Sums do not match"
    return fu_2



import argparse
import os

import pytest

import parsl

from concurrent.futures import Future
from parsl import File
from parsl.app.app import bash_app


@bash_app
def app1(inputs=[], outputs=[], stdout=None, stderr=None, mock=False):
    cmd_line = f"""echo 'test' > {outputs[0]}"""
    return cmd_line


@bash_app
def app2(inputs=[], outputs=[], stdout=None, stderr=None, mock=False):

    with open('somefile.txt', 'w') as f:
        f.write("%s\n" % inputs[0])
    cmd_line = f"""echo '{inputs[0]}' > {outputs[0]}"""
    return cmd_line


def test_behavior():
    app1_future = app1(inputs=[],
                       outputs=[File("simple-out.txt")])

    o = app1_future.outputs[0]
    assert isinstance(o, Future)

    app2_future = app2(inputs=[o],
                       outputs=[File("simple-out2.txt")])
    app2_future.result()

    expected_name = 'b'
    with open('somefile.txt', 'r') as f:
        name = f.read()

    with open(app2_future.outputs[0].filepath, 'r') as f:
        expected_name = f.read()

    assert name == expected_name, "Filename mangled due to DataFuture handling"



import argparse
import os
import pytest
import shutil

from concurrent.futures import wait

from parsl import File, python_app
from parsl.tests.configs.local_threads import fresh_config as local_config


@python_app
def double(x, outputs=[]):
    with open(outputs[0], 'w') as f:
        f.write(x * 5)
    return x * 5


whitelist = os.path.join(os.path.dirname(os.path.dirname(__file__)), 'configs', '*threads*')


@pytest.mark.issue363
def test_launch_apps(n=2, outdir='outputs'):
    if not os.path.exists(outdir):
        os.makedirs(outdir)
    else:
        shutil.rmtree(outdir)
        os.makedirs(outdir)
    print('outdir is ', outdir)

    all_futs = []
    for i in range(n):
        fus = double(i, outputs=[File('{0}/{1}.txt'.format(outdir, i))])
        all_futs.append(fus)

    wait(all_futs)

    stdout_file_count = len(
        [item for item in os.listdir(outdir) if item.endswith('.txt')])
    assert stdout_file_count == n, "Only {}/{} files in '{}' ".format(
            len(os.listdir('outputs/')), n, os.listdir(outdir))
